@@ -47,3 +47,19 @@ class TestTasks:
 
         error = exc_info.value
         assert error.status_code == 400
+
+    @pytest.mark.vcr
+    def test_get_tasks_filters_valid(self, grocy):
+        query_filter = ["category_id=1"]
+        tasks = grocy.tasks(query_filters=query_filter)
+
+        for item in tasks:
+            assert item.category_id == 1
+
+    @pytest.mark.vcr
+    def test_get_tasks_filters_invalid(self, grocy, invalid_query_filter):
+        with pytest.raises(GrocyError) as exc_info:
+            grocy.tasks(query_filters=invalid_query_filter)
+
+        error = exc_info.value
+        assert error.status_code == 500
